% Linearisation of model III.A in
% Behar et al,Biophysical Journal 93(2007): 806-821
close all
clear all

% State variables
syms K2s Ks Ps s

k1=1e-1;
k1m=1.5e-1;
V2=3e-3;
k2m=1;
k3=4.8e-4;
k3m=1;
V4=5e-6;
k4m=1;
k5=3e-2;
%k5=10*3e-2;
k5m=2e-3;
k6=10;
k6m=0.1;
V7=1;
k7m=1e-1;

f1 = k1*s*(1-K2s)/(k1m+(1-K2s))-V2*K2s/(k2m+K2s)-k5*Ps*K2s/(k5m+K2s);
f2 = k6*K2s*(1-Ks)/(k6m+(1-Ks))-V7*Ks/(k7m+Ks);
f3 = k3*Ks*(1-Ps)/(k3m+(1-Ps))-V4*Ps/(k4m+Ps);

% Jacobian matrix
Jac_fx=jacobian([f1;f2;f3],[K2s,Ks,Ps]);

% Compute the equilibrium point by simulating the nonlinear system
% Stimulus amplitude
s_in=0.1;
Tfin=0.3*60*60;
[tv,xv]=ode23s(@behar3A,linspace(0,1000*Tfin,401),[0,0,0],[],s_in);
xe=xv(end,:);
plot(tv,xv(:,2))

% Compute dynamical matrix A at equilibrium point 0
A_lin=subs(Jac_fx,[K2s,Ks,Ps,s],[xe,s_in]);
% Compute input matrix of the linearised system
Jac_fu=jacobian([f1;f2;f3],[s]);
B_lin=subs(Jac_fu,[K2s,Ks,Ps,s],[xe,s_in]);

% Build the linearised system 
C_lin=[0,1,0];
D_lin=0;

behar3A_lin=ss(A_lin,B_lin,C_lin,D_lin);
display(['dcgain: ', num2str(dcgain(behar3A_lin))])
behar3A_lin_tf=tf(behar3A_lin);
display('Poles: ')
pole(behar3A_lin_tf)
%[yv2,tv2]=step(2*s_in*behar3A_lin,Tfin);
%figure
%plot(tv2,yv2)