% Analysis of the I/O characteristic of the three modules of the MAPK
% cascade and of the whole open-loop system

clear all
clc

% Parameters definition
global ytot ztot V0 V1 V2 V3 V4 V5 V6 V7 V8 V9 V10
global K2 K3 K4 K5 K6 K7 K8 K9 K10
ytot=1200;          % [nM]
ztot=300;           % [nM]
V0=0.0015;          % [1/(nM * s)]
V1=2E-6;            % [1/s]
V2=1.2;             % [nM * 1/s]
K2=200;             % [nM]
V3=0.064;           % [1/s]
K3=1200;            % [nM]
V4=0.064;           % [1/s]
K4=1200;            % [nM]
V5=5;               % [nM * 1/s]
K5=1200;            % [nM]
V6=5;               % [nM * 1/s]
K6=1200;            % [nM]
V7=0.06;            % [1/s]
K7=300;             % [nM]
V8=0.06;            % [1/s]
K8=300;             % [nM]
V9=5;               % [nM * 1/s]
K9=300;             % [nM]
V10=5;              % [nM * 1/s]
K10=300;            % [nM]

%% Compute the steady-state IO characteristic of module 1 for different 
% (constant) values of the input w

% Number of input values
Np=101;

% Vector of input values
wv = linspace(0,300,Np);
% Time span
tv1=linspace(0,6000,301);
% Initial condition
x0=0;

% Vector of steady-state values
xss_v=zeros(Np,1);
for iw = 1:length(wv)
    [tsol,xv1] = ode45(@IO_module1,tv1,x0,[],wv(iw));
    xss_v(iw)=xv1(end);
end

figure(1)
plot(wv,xss_v)

%% Compute the steady-state IO characteristic of module 2 for different 
% (constant) values of the input x

% Vector of input values
xv = linspace(0,300,Np);
% Initial condition
y0=[0;0];

% Vector of steady-state values
yss_v=zeros(Np,1);
for ix = 1:length(xv)
    [tsol,ysol] = ode45(@IO_module2,tv1,y0,[],xv(ix));
    yss_v(ix)=ysol(end,2);
end

figure(2)
plot(xv,yss_v)


%% Compute the steady-state IO characteristic of module 3 for different 
% (constant) values of the input y3

% Vector of input values
yv = linspace(0,500,Np);
% Initial condition
z0=[0;0];

% Vector of steady-state values
zss_v=zeros(Np,1);
for iy = 1:length(yv)
    [tsol,zsol] = ode45(@IO_module3,tv1,z0,[],yv(iy));
    zss_v(iy)=zsol(end,2);
end

figure(3)
plot(yv,zss_v)


%% Compute the steady-state IO characteristic of the full open-loop system

% Initial condition
states0=zeros(5,1);

% Vector of steady-state values
zss_v2=zeros(Np,1);
for iw = 1:length(wv)
    [tsol,zsol2] = ode45(@MAPK_Model_OL,tv1,states0,[],wv(iw));    
    zss_v2(iw)=zsol2(end,5);
end

figure(4)
plot(wv,zss_v2,wv,wv)


%% Compute the trajectory of the closed-loop system from several random
% initial conditions

% Number of trajectories
Ntr=4;

% Matrix to store the vectors of initial conditions
%init_cond=[rand(1,Ntr)*80; zeros(1,Ntr); rand(1,Ntr)*100; zeros(1,Ntr); rand(1,Ntr)*70];
%init_cond = [linspace(0,30,Ntr); zeros(1,Ntr); linspace(0,1200,Ntr); zeros(1,Ntr); linspace(0,50,Ntr)];
%init_cond = [linspace(0,120,Ntr); zeros(1,Ntr); linspace(0,1200,Ntr); zeros(1,Ntr); linspace(0,300,Ntr)];
init_cond=[10  20   70   100;
            0  1050 1000 0;
            10 150  200  200;
            0  250  200    0;
            10 50   100  170];

% Ntr=1;
% init_cond=[10;0;20;0;100];

clear tsol xsol
for itr = 1:Ntr
    [tsol(:,itr),xsol(:,:,itr)] = ode23(@MAPK_Model,tv1,init_cond(:,itr),[],1);
end
figure(5)
clf
% set(0,'DefaultAxesColorOrder',[0 0 1],...
%        'DefaultAxesLineStyleOrder','-|--|.|-o|--s|:x')
 set(0,'DefaultAxesColorOrder',[0 0 1],...
        'DefaultAxesLineStyleOrder','-|-|-|-|-|-')
subplot(221)
hold on
plot(tsol,squeeze(xsol(:,1,:)),'LineWidth',0.5,'Marker','none')
%plot(tsol(1:10:end,:),squeeze(xsol(1:10:end,1,:)),'LineStyle','none','MarkerSize',5)
axis([-20,6000,-5,110])
xlabel('Time [s]')
ylabel('Mos [nM]')
subplot(222)
hold on
xlabel('Time [s]')
ylabel('MEK-PP [nM]')
plot(tsol,squeeze(xsol(:,3,:)),'LineWidth',0.5,'Marker','none')
%plot(tsol(1:10:end,:),squeeze(xsol(1:10:end,3,:)),'LineStyle','none','MarkerSize',5)
axis([-20,6000,-30,600])
subplot(223)
hold on
xlabel('Time [s]')
ylabel('p42-PP [nM]')
plot(tsol,squeeze(xsol(:,5,:)),'LineWidth',0.5,'Marker','none')
%plot(tsol(1:10:end,:),squeeze(xsol(1:10:end,5,:)),'LineStyle','none','MarkerSize',5)
axis([-20,6000,-20,300])