% Model of the MAPK cascade studied in (Angeli, Ferrell, Sontag, PNAS 2004)
%
% The model has been modified, in particular the differential equation of x
% in the original model includes a term V0*w*x, which has been changed to
% V0*w in the present model.
% In the original model, indeed, the first module is unstable (there are no
% nonnegative equilibrium points).

function dstate_v = MAPK_Model_OL(t,state_v,w)

% Definition of the state variables
x=state_v(1);
y1=state_v(2);
y3=state_v(3);
z1=state_v(4);
z3=state_v(5);

% Parameters definition
ytot=1200;          % [nM]
ztot=300;           % [nM]
V0=0.0015;          % [1/(nM * s)]
V1=2E-6;            % [1/s]
V2=1.2;             % [nM * 1/s]
K2=200;             % [nM]
V3=0.064;           % [1/s]
K3=1200;            % [nM]
V4=0.064;           % [1/s]
K4=1200;            % [nM]
V5=5;               % [nM * 1/s]
K5=1200;            % [nM]
V6=5;               % [nM * 1/s]
K6=1200;            % [nM]
V7=0.06;            % [1/s]
K7=300;             % [nM]
V8=0.06;            % [1/s]
K8=300;             % [nM]
V9=5;               % [nM * 1/s]
K9=300;             % [nM]
V10=5;              % [nM * 1/s]
K10=300;            % [nM]

%% Equations of the model
% Conservation equations
y2 = ytot - y1 - y3;
z2 = ztot - z1 - z3;
% Differential equations
%dx  = -V2*x/(K2+x) + V0*w*x + V1;
dx  = -V2*x/(K2+x) + V0*w + V1;
dy1 = V6*y2/(K6+y2) - V3*x*y1/(K3+y1);
dy3 = V4*x*y2/(K4+y2) - V5*y3/(K5+y3);
dz1 = V10*z2/(K10+z2) - V7*y3*z1/(K7+z1);
dz3 = V8*y3*z2/(K8+z2) - V9*z3/(K9+z3);
%dz3 = V8*y3*z2/(K8+z2) - V9*z3/(K9+z3) + V1*2e5;

dstate_v = [dx; dy1; dy3; dz1; dz3];
