% Example of Least Squares identification
%
%%%%%%%%% Redo the identification experiment on the static mapping
%%%%%%%%%                       y = A x + B u
% where the matrices A,B are the same of the dynamical system
%Nsteps=13;

load sys4

% Do Nexp identification experiments on the same matrix, to average the 
% effect of noise
Nexp=100;

% Number of steps for the dynamical system simulation
% In the static system identification the number of points is Nsteps-1
Nsteps=21;

noise_lev=0.05;

% Ridge regression parameter
%a=0.02;

% Vectors of quality index
AUROC=zeros(Nexp,1);
AUPR=zeros(Nexp,1);

% Array containing normalized matrices for each experiment
Anorm=zeros(Nexp,Nx,Nx);

% Array containing edgerankings for each experiment
edgeranking=zeros(Nexp,Nx^2,3);

% Arrays of sensitivity and comlementary specificity values for each experiment
Sens=zeros(Nexp,Nx^2-Nx);
Compl_Spec=zeros(Nexp,Nx^2-Nx);

for iexp=1:Nexp
    
    % Define the X values
    Xsm=[];
    Ysm=[];
    for i=1:Nsteps-1
        x_sm=rand(1,Nx);
        Xsm=[Xsm; x_sm];
        Ysm=[Ysm; (sys.a*x_sm'+sys.b*1)'];
    end
    
    % Define the matrices for identification
    Xnext=Ysm';
    Omega=[Xsm';ones(1,Nsteps-1)];
    
    % Solve the problem Omega' * Theta' = Xnext'
    Theta=Omega'\Xnext';
    
    Ahat=Theta(1:Nx,1:Nx)';
    Bhat=Theta(Nx+1,:)';
    
    
    
    
    % Add some noise to the data
    Xnext_n = (Ysm+noise_lev*randn(Nsteps-1,Nx))';
    
    % Repeat the identification with the noisy data
    
    % Define the matrices for identification
    Omega_n=Omega;
    
    % Solve the problem Omega' * Theta' = Xnext'
    Theta_n=Omega_n'\Xnext_n';
    Ahat_n=Theta_n(1:Nx,1:Nx)';
    Bhat_n=Theta_n(Nx+1,:)';
    
    % Ridge regression
    %Theta_r=(Omega_n*Omega_n'+a*eye(Nx+1))\Omega_n*Xnext_n';
    %Ahat_r=Theta_r(1:Nx,1:Nx)';
    %Bhat_r=Theta_r(Nx+1,:)';
    
    
    % Normalize the adjacency matrix for standard least squares
    for i=1:Nx
        for j=1:Nx
            Anorm(iexp,i,j)=Ahat_n(i,j)/sqrt(norm(Ahat_n(i,:),inf)*norm(Ahat_n(:,j),inf));
        end
    end
    
    %     % Normalize the adjacency matrix from rigde regression
    %     Anorm_r=zeros(Nx);
    %     for i=1:Nx
    %         for j=1:Nx
    %             Anorm_r(i,j)=Ahat_r(i,j)/sqrt(norm(Ahat_r(i,:),inf)*norm(Ahat_r(:,j),inf));
    %         end
    %     end
    
    % Write the ranking of the nodes
    for i=1:Nx
        for j=1:Nx
            if i~=j
                edgeranking(iexp,(Nx*(i-1))+j,:)=[abs(Anorm(iexp,i,j)),i,j];
            else
                edgeranking(iexp,(Nx*(i-1))+j,:)=[0,i,j];
            end
        end
    end
    % Sort the ranking list in descending order with respect to the absolute
    % value of the edge weight
    edgeranking(iexp,:,:)=sortrows(squeeze(edgeranking(iexp,:,:)),-1);
    
    % Compute data for the ROC curve
    % Total number of Positives and Negatives
    Tot_Neg=sum(sum((sys.A==0)));
    Tot_Pos=Nx^2-Nx-Tot_Neg;
    
    % Compute sensitivity and specificity for standard least squares
    for i=1:Nx^2-Nx
        TP(i)=0; FP(i)=0;
        % Compute the sensitivity for the partial node list
        for j=1:i
            if sys.a(edgeranking(iexp,j,2),edgeranking(iexp,j,3))==0
                FP(i)=FP(i)+1;
            else
                TP(i)=TP(i)+1;
            end
        end
        TN(i)=Tot_Neg-FP(i);
        FN(i)=Tot_Pos-TP(i);
        
        Sens(iexp,i)=TP(i)/(Tot_Pos);
        Compl_Spec(iexp,i)=1-TN(i)/(Tot_Neg);
    end
    
    % % Recompute the ROC curve for the ridge regression data
    % edgeranking_r=zeros(Nx^2,3);
    % for i=1:Nx
    %     for j=1:Nx
    %         if i~=j
    %             edgeranking_r((Nx*(i-1))+j,:)=[abs(Anorm_r(i,j)),i,j];
    %         else
    %             edgeranking_r((Nx*(i-1))+j,:)=[0,i,j];
    %         end
    %     end
    % end
    % edgeranking_r=sortrows(edgeranking_r,-1);
    %
    % for i=1:Nx^2-Nx
    %     TP(i)=0; FP(i)=0;
    %     % Compute the sensitivity for the partial node list
    %     for j=1:i
    %         if sys.a(edgeranking_r(j,2),edgeranking_r(j,3))==0
    %             FP(i)=FP(i)+1;
    %         else
    %             TP(i)=TP(i)+1;
    %         end
    %     end
    %     TN(i)=Tot_Neg-FP(i);
    %     FN(i)=Tot_Pos-TP(i);
    %
    %     Sens_r(i)=TP(i)/(Tot_Pos);
    %     Spec_r(i)=TN(i)/(Tot_Neg);
    %     Compl_Spec_r(i)=1-Spec(i);
    % end
    
end

% Compute the median and standard deviation of each entry (absolute value)
% in the identified normalized matrices
median_v=zeros(Nx,Nx);
stdev_v=zeros(Nx,Nx);
for i=1:Nx
    for j=1:Nx
        % Compute the median of the current i,j element
        median_v(i,j)=median(abs(Anorm(:,i,j)));
        stdev_v(i,j)=std(abs(Anorm(:,i,j)));
    end
end

% Also compute the median sensitivity and complementary specificity and the
% corresponding standard deviation
median_Sens=median(Sens);
stdev_Sens=std(Sens);
median_Compl_Spec=median(Compl_Spec);
stdev_Compl_Spec=std(Compl_Spec);

% Plot the median ROC curve
figure(2)
clf
plot(median_Compl_Spec,median_Sens,'b','LineWidth',2)
xlabel('1-specificity (median)')
ylabel('sensitivity (median)')
hold
plot([0,1],[0,1],'r--')

% Compare reconstructed vs original networks using colormaps
figure(2)
subplot(2,2,1)
imagesc(abs(sys.a))
colormap(1-gray)
colorbar
title('Original network')
subplot(2,2,2)
imagesc(median_v)
colormap(1-gray)
colorbar
title(['LS estimate (\sigma = ', num2str(noise_lev), ')'])
% subplot(2,2,3)
% imagesc(median_v)
% colormap(1-gray)
% colorbar
% title(['LS estimate (\sigma = ', num2str(noise_lev), ')'])
subplot(2,2,4)
imagesc(median_v)
colormap(1-gray)
colorbar
title(['LS estimate (\sigma = ', num2str(noise_lev), ')'])

% Draw the boxplots for the optimisation parameters
Anorm_resh=abs(reshape(Anorm,100,16));
Anorm_resh=Anorm_resh(:,[3,5,7,8,13,14,2,4,9,10,12,15]);
figure(3)
clf
h_axes=axes;
boxplot(Anorm_resh)
set(gca,'XTick',[1:12],'XTickLabel','(3,1)|(1,2)|(3,2)|(4,2)|(1,4)|(2,4)|(2,1)|(4,1)|(1,3)|(2,3)|(4,3)|(3,4)');
title(['Noise \sigma = ', num2str(noise_lev)])