clear all
clc

% Generate a stable random discrete-time system
% Number of states
Nx=5;

% Do Nexp identification experiments on the same matrix, to average the 
% effect of noise
Nexp=100;

% Sparseness coefficient
spars_coeff=0.5;

sys=rss(Nx,1,1);

% Make the random A matrix sparse
%
% Number of elements to set to zero
Nzero=ceil(spars_coeff*(Nx^2-Nx));
for i=1:Nzero
    exit_loop=false;
    while not(exit_loop)
        % Take a random positive value
        rand_val=abs(4*rand);
        % Find the element in the matrix whose abs is closer to the random
        % value
        [var1,row_idx]=min(min(abs(abs(sys.a)-rand_val),[],2));
        [var1,col_idx]=min(min(abs(abs(sys.a)-rand_val),[],1));

        % Set the value to zero, but only if the element is not on the diagonal
        % and the element has not been already previously set to zero
        if row_idx~=col_idx && sys.a(row_idx,col_idx)~=0
            sys.a(row_idx,col_idx)=0;
            exit_loop=true;
        end
    end
end

% Take the eigenvalues to the left-half plane
max_eig_real=abs(max(real(eig(sys.a))));
if max_eig_real>=0
    sys.a=sys.a-(max_eig_real+0.3)*eye(Nx);
end

% Load the system used in the book
load discr_vs_cont

% Show the A matrix and the pzmap
figure(1)
sys.a
pzmap(sys)

% Compute the sampling time Ts
% First compute the fastest eigenvalue absolute value
% min_eig_real=abs(min(real(eig(sys.a))));
% Ts1=1/10/min_eig_real;
% Ts2=1/5/min_eig_real;
% Ts3=1/2/min_eig_real;

figure(2)
subplot(3,2,1)
[yv,tv,xv]=step(sys);
plot(tv,xv)
title('Continuous-time')


% Settling time of the slowest eigenvalue
max_eig_real=abs(max(real(eig(sys.a))));
Ta=4.6/max_eig_real;
% Sampling time Ts
Ts=[1/50*Ta; 1/20*Ta; 1/10*Ta; 1/5*Ta; 1/3*Ta];

for i=1:length(Ts)
    subplot(3,2,i+1)
    dsys{i}=c2d(sys,Ts(i),'zoh');
    [yvd,tvd,xvd]=step(dsys{i});
    plot(tvd,xvd)
    title(['T_s = T_a / ', num2str(round(Ta/Ts(i)))])
end

% Compare reconstructed vs original networks using colormaps
% Normalise the matrices
Ad_norm=cell(length(Ts)+1,1);
for i=1:Nx
    for j=1:Nx
        Ad_norm{1}(i,j)=sys.a(i,j)/sqrt(norm(sys.a(i,:),inf)*norm(sys.a(:,j),inf));
    end
end
for km=2:length(Ts)+1
    for i=1:Nx
        for j=1:Nx
            Ad_norm{km}(i,j)=dsys{km-1}.a(i,j)/sqrt(norm(dsys{km-1}.a(i,:),inf)*norm(dsys{km-1}.a(:,j),inf));
        end
    end
end
figure(3)
subplot(3,2,1)
imagesc(abs(Ad_norm{1}))
colormap(1-gray)
colorbar
title('A (continuous-time)')
for i=1:length(Ts)
    subplot(3,2,i+1)
    imagesc(abs(Ad_norm{i+1}),[0 1])
    %imagesc(abs(A1-diag(diag(A1))))
    colormap(1-gray)
    colorbar
    title(['T_s = T_a / ', num2str(round(Ta/Ts(i)))])
end